function _TDVPUpdate2(H::SparseProjectiveHamiltonian{2}, Al::MPSTensor, Ar::MPSTensor, dt::Number, alg::KrylovKit.KrylovAlgorithm; kwargs...)

     reset_timer!(get_timer("action2"))
     expx, info = _LanczosExp(x -> action2(H, x; kwargs...),
          dt,
          CompositeMPSTensor(Al, Ar),
          alg)

     # normalize
     Norm = norm(expx)
     rmul!(expx, 1 / Norm)
     return expx, Norm, info

end

function _TDVPUpdate1(H::SparseProjectiveHamiltonian{1}, A::MPSTensor, dt::Number, alg::KrylovKit.KrylovAlgorithm; kwargs...)

     reset_timer!(get_timer("action1"))

     if get(kwargs, :prefuse, false)
          PH = _prefuse(H)
          expx, info = _LanczosExp(x -> action1(PH, x; kwargs...),
               dt,
               A,
               alg)
     else
          expx, info = _LanczosExp(x -> action1(H, x; kwargs...),
               dt,
               A,
               alg)
     end

     # normalize
     Norm = norm(expx)
     rmul!(expx, 1 / Norm)
     return expx, Norm, info

end

function _TDVPUpdate0(H::SparseProjectiveHamiltonian{0}, A::MPSTensor{2}, dt::Number, alg::KrylovKit.KrylovAlgorithm; kwargs...)

     reset_timer!(get_timer("action0"))
     expx, info = _LanczosExp(x -> action0(H, x; kwargs...),
          dt,
          A,
          alg)

     # normalize
     Norm = norm(expx)
     rmul!(expx, 1 / Norm)
     return expx, Norm, info

end

function _LanczosExp(f, t::Number, x, args...; kwargs...)
     # wrap KrylovKit.exponentiate to make sure the integrate step length is exactly t
     # TODO, unknown bugs if t is too large

     x, info = exponentiate(f, t, x, args...; kwargs...)
     residual = info.residual
     K_sum = info.numops
     while residual != 0
          dt = sign(t) * residual
          x, info = exponentiate(f, dt, x, args...; kwargs...)
          residual = info.residual
          K_sum += info.numops
     end
     info = LanczosInfo(info.converged > 0, [info.normres,], info.numiter, K_sum)
     return x, info
end